/* MySQLLoginPanel.m created by dmorin on Thu 22-Jul-1999 */
#ifndef PDO
#import "MySQLLoginPanel.h"
#import "MySQLDescription.h"


@implementation MySQLLoginPanel

extern char *MySQL_VERS_NUM[];

- init
{
    [super init];
    [NSBundle loadNibNamed:@"MySQLLoginPanel" owner:self];

    [versionField setStringValue:[NSString stringWithFormat:[versionField stringValue], MySQL_VERS_NUM]];
    return self;
}

- (void)dealloc
{
    [databasesNameArray release];
    [adaptor release];
    [super dealloc];
}

- (NSDictionary *)runPanelForAdaptor:(EOAdaptor *)anAdaptor validate:(BOOL)validate
{
    NSDictionary *result;
    BOOL success;

    adaptor = [anAdaptor retain];
    [self setConnectionDictionary:[adaptor connectionDictionary]];
    do {
        if ([NSApp runModalForWindow:[userNameField window]] != NSRunContinuesResponse) {
            result = nil;
            [[userNameField window] orderOut:nil];
            break;
        }
        result = [self connectionDictionary];
        [[userNameField window] orderOut:nil];

        if (!validate) break;

        success = YES;
        [adaptor setConnectionDictionary:result];
        NS_DURING
            [adaptor assertConnectionDictionaryIsValid];
        NS_HANDLER
            NSRunAlertPanel (NULL, @"%@", NULL, NULL, NULL, [localException reason]);
            [self setConnectionDictionary:[adaptor connectionDictionary]];
            success = NO;
        NS_ENDHANDLER
    } while (!success);
    return result;
}

- (void)cancel:sender
{
    [NSApp stopModalWithCode:NSRunAbortedResponse];
}

- (void)login:sender
{
    [NSApp stopModalWithCode:NSRunContinuesResponse];
}

- (void)browseDatabases:sender
{
    [databasesNameArray release];

    NS_DURING
        databasesNameArray = [[adaptor describeDatabaseNamesWithConnectionDictionary:[self connectionDictionary]] retain];
    NS_HANDLER
        databasesNameArray = nil;
    NS_ENDHANDLER
    
    [databaseBrowser reloadData];
    [databaseBrowser sizeLastColumnToFit];

    [self databaseNameChanged:sender]; //select the right row
}

- (void)selectDatabase:sender
{
    [databaseNameField setStringValue:[databasesNameArray objectAtIndex:[databaseBrowser selectedRow]]];
}

- (void)databaseNameChanged:sender
{
    NSString *name = [databaseNameField stringValue];

    if ( [databasesNameArray containsObject:name] )
        [databaseBrowser selectRow:[databasesNameArray indexOfObject:name] byExtendingSelection:NO];
    else
        [databaseBrowser deselectAll:sender];
}

- (void)textDidChange:(NSNotification *)aNotification
{
    [browseButton setEnabled:YES];
}

- (void)setConnectionDictionary:(NSDictionary *)connectionInfo
{
    NSString *hostName, *port, *userName, *password, *databaseName, *useMySQLStore;

    hostName = [connectionInfo objectForKey:@"hostName"];
    if (!hostName)
        hostName = @"";
    [hostNameField setStringValue:hostName];

    port = [connectionInfo objectForKey:@"port"];
    if (!port)
        port = @"";
    [portField setStringValue:port];

    userName = [connectionInfo objectForKey:@"userName"];
    if (!userName)
        userName = @"";
    [userNameField setStringValue:userName];

    password = [connectionInfo objectForKey:@"password"];
    if (!password)
        password = @"";
    [passwordField setStringValue:password];

    databaseName = [connectionInfo objectForKey:@"databaseName"];
    if (!databaseName)
        databaseName = @"";
    [databaseNameField setStringValue:databaseName];

    useMySQLStore = [connectionInfo objectForKey:@"useMySQLStoreResult"];
    [useMySQLStoreButton setState:[useMySQLStore isEqualToString:@"YES"]];
}

- (NSDictionary *)connectionDictionary
{
    NSString *hostName, *port, *userName, *password, *databaseName, *useMySQLStore;
    NSMutableDictionary *info;
    
    info = [NSMutableDictionary dictionaryWithCapacity:6];

    hostName =  [hostNameField stringValue];
    if (!hostName)
        hostName = @"";
    [info setObject:hostName forKey:@"hostName"];

    port =  [portField stringValue];
    if (!port)
        port = @"";
    [info setObject:port forKey:@"port"];

    userName =  [userNameField stringValue];
    if (!userName)
        userName = @"";
    [info setObject:userName forKey:@"userName"];

    password =  [passwordField stringValue];
    if (!password)
        password = @"";
    [info setObject:password forKey:@"password"];

    databaseName =  [databaseNameField stringValue];
    if (!databaseName)
        databaseName = @"";
    [info setObject:databaseName forKey:@"databaseName"];

    [info setObject:@"ISO Latin-1" forKey:@"databaseEncoding"];

    useMySQLStore = [useMySQLStoreButton state] ? @"YES" : @"NO";
    [info setObject:useMySQLStore forKey:@"useMySQLStoreResult"];

    return info;
}

- (int)numberOfRowsInTableView:(NSTableView *)theTableView
{
    return [databasesNameArray count];
}

- (id)tableView:(NSTableView *)theTableView objectValueForTableColumn:(NSTableColumn *)theColumn row:(int)rowIndex
{
    if ([[theColumn identifier] isEqualToString:@"database"])
        return [databasesNameArray objectAtIndex:rowIndex];
    else
        return nil;
}

@end
#endif