/*
 * This file is subject to the MiscKit license, see http://www.misckit.org/
 * and search around for details. The bottom line is: you may do what you please
 * with this code, but you're on your own if you run into trouble...
 * Copyright (C) 1999,2000,2001 dmorin,anjo@mac.com
 * Portions of this code derived from the FlatFileAdaptor (C) Apple Computer
 */

#import "MySQLAdaptor+ExternalTypeMapping.h"
#import "MySQLChannel.h"

@implementation MySQLAdaptor (ExternalTypeMapping)

static NSString *externalTypeNames[] = {
    @"TINYINT", @"SMALLINT", @"INT", @"MEDIUMINT", @"BIGINT",
    @"DECIMAL", @"FLOAT", @"DOUBLE", @"TIMESTAMP", @"DATE",
    @"TIME", @"DATETIME", @"YEAR", @"VARCHAR", @"TINYTEXT",
    @"TEXT", @"MEDIUMTEXT", @"LONGTEXT", @"TINYBLOB", @"BLOB",
    @"MEDIUMBLOB", @"LONGBLOB", @"ENUM", @"SET", @"CHAR", nil };

static NSString *internalTypeNames[] = {
    @"NSNumber", @"NSNumber", @"NSNumber", @"NSNumber", @"NSNumber",
    @"NSDecimalNumber", @"NSDecimalNumber", @"NSDecimalNumber", @"NSCalendarDate", @"NSCalendarDate",
    @"NSCalendarDate", @"NSCalendarDate", @"NSString", @"NSString", @"NSString",
    @"NSString", @"NSString", @"NSString", @"NSData", @"NSData",
    @"NSData", @"NSData", @"NSString", @"NSString", @"NSString", nil };

+ (NSDictionary *)externalToInternalTypeMap
{
    static NSDictionary *externalToInternalTypeMap = nil;

    if (!externalToInternalTypeMap) {
        int i;
        for (i = 0; externalTypeNames[i]; i++);
        externalToInternalTypeMap = [[NSDictionary dictionaryWithObjects:internalTypeNames forKeys:externalTypeNames count:i] retain];
    }

    return externalToInternalTypeMap;
}

+ (NSString *)internalTypeForExternalType:(NSString *)extType model:(EOModel *)model
{
    return [[self externalToInternalTypeMap] objectForKey:extType];
}

+ (NSArray *)externalTypesWithModel:(EOModel *)model
{
    return [[self externalToInternalTypeMap] allKeys];
}

+ (NSString *)defaultExternalType
{
    return externalTypeNames[1];
}

// ak: renamed from assignExternalInfoForAttribute
// I hope this is solely used for switching models...
+ (void)assignExternalTypeForAttribute:(EOAttribute *)attribute
{
    EOAdaptorValueType valueType = [attribute adaptorValueType];
    NSString *externalType = externalTypeNames[0], *currentType = [attribute externalType];

    //ak: this should take care of all cases where the current Type is known
    if(currentType && [[self externalToInternalTypeMap] objectForKey:currentType] != nil)
        return;
    switch(valueType)
    {
        case EOAdaptorNumberType:
            if([[attribute valueType] isEqualToString:@"i"])
            {
                externalType = @"INT";
                //ak: this is needed for the Movies sample...
                [attribute setPrecision:0];
                [attribute setScale:0];
            }
            else if([[attribute valueType] isEqualToString:@"f"])
                externalType = @"DECIMAL";
            else
                externalType = @"DOUBLE";
            break;
        case EOAdaptorCharactersType:
            if([attribute width] <= 0)
                externalType = @"TEXT";
            else
                externalType = @"VARCHAR";
            break;
        case EOAdaptorBytesType:
            externalType = @"BLOB";
            break;
        case EOAdaptorDateType:
            externalType = @"DATETIME";
            break;
        default:
            externalType = @"INT";
            break;
    }
    [attribute setExternalType:externalType];
}

+ (void)assignExternalInfoForAttribute:(EOAttribute *)attribute
{
    [super assignExternalInfoForAttribute:attribute];
    //if([attribute columnName])
    //   [attribute setColumnName:[NSString stringWithFormat:@"%@_", [attribute columnName]]];
}

+ (void)assignExternalInfoForEntity:(EOEntity *)entity
{
    NSEnumerator *attributeEnumerator = [[entity attributes] objectEnumerator];
    EOAttribute *attribute;

    while (attribute = [attributeEnumerator nextObject])
        [self assignExternalInfoForAttribute:attribute];
}

+ (void)assignExternalInfoForEntireModel:(EOModel *)model
{
    NSEnumerator *entityEnumerator = [[model entities] objectEnumerator];
    EOEntity *entity;

    while (entity = [entityEnumerator nextObject])
        [self assignExternalInfoForEntity:entity];
}

+ (NSString *)externalTypeForMySQLType:(int)i
{
    switch (i)
    {
        case FIELD_TYPE_DECIMAL:
            return externalTypeNames[5];
        case FIELD_TYPE_TINY:
            return externalTypeNames[0];
        case FIELD_TYPE_SHORT:
            return externalTypeNames[1];
        case FIELD_TYPE_LONG:
            return externalTypeNames[2];
        case FIELD_TYPE_FLOAT:
            return externalTypeNames[6];
        case FIELD_TYPE_DOUBLE:
            return externalTypeNames[7];
        case FIELD_TYPE_NULL:
            return externalTypeNames[13];
        case FIELD_TYPE_TIMESTAMP:
            return externalTypeNames[8];
        case FIELD_TYPE_LONGLONG:
            return externalTypeNames[4];
        case FIELD_TYPE_INT24:
            return externalTypeNames[3];
        case FIELD_TYPE_DATE:
            return externalTypeNames[9];
        case FIELD_TYPE_TIME:
            return externalTypeNames[10];
        case FIELD_TYPE_DATETIME:
            return externalTypeNames[11];
        case FIELD_TYPE_YEAR:
            return externalTypeNames[12];
        case FIELD_TYPE_NEWDATE:
            return externalTypeNames[9];
        case FIELD_TYPE_ENUM:
            return externalTypeNames[23];
        case FIELD_TYPE_SET:
            return externalTypeNames[24];
        case FIELD_TYPE_TINY_BLOB:
            return externalTypeNames[18];
        case FIELD_TYPE_MEDIUM_BLOB:
            return externalTypeNames[20];
        case FIELD_TYPE_LONG_BLOB:
            return externalTypeNames[21];
        case FIELD_TYPE_BLOB:
            return externalTypeNames[19];
        case FIELD_TYPE_VAR_STRING:
            return externalTypeNames[13];
        case FIELD_TYPE_STRING:
            return externalTypeNames[24];
    }
    return [MySQLAdaptor defaultExternalType];
}

@end