/*
 * This file is subject to the MiscKit license, see http://www.misckit.org/
 * and search around for details. The bottom line is: you may do what you please
 * with this code, but you're on your own if you run into trouble...
 * Copyright (C) 1999,2000,2001 dmorin,anjo@mac.com
 * Portions of this code derived from the FlatFileAdaptor (C) Apple Computer
 */

#import "MySQLAdaptor.h"

#import "MySQLContext.h"
#import "MySQLChannel.h"
#import "MySQLDescription.h"
#import "MySQLSQLExpression.h"


@implementation MySQLAdaptor

+ (NSDictionary *)defaultConnectionDictionary
{
    static NSDictionary *dict = nil;

    if (!dict)
        dict = [[NSDictionary dictionaryWithObjectsAndKeys:
            @"ISO Latin-1", @"databaseEncoding",
            @"", @"userName",
            @"", @"password",
            @"localhost", @"hostName",
            @"3306", @"port",
            @"Test", @"databaseName",
            @"NO", @"useMySQLStoreResult",
            nil] retain];

    return dict;
}

- initWithName:(NSString *)name
{
    //debugEnabled = [[NSUserDefaults standardUserDefaults] boolForKey:@"EODebuggingEnabled"];

    // if (debugEnabled) NSLog (@" *** [%@ initWithName]\n", self);	// added dec 7th...

    [super initWithName:name];
    [self setConnectionDictionary:[[self class] defaultConnectionDictionary]];

    return self;
}

- createAdaptorContext
{
    return [[[MySQLContext alloc] initWithMySQLAdaptor:self] autorelease];
}

- (Class)expressionClass
{
    return [MySQLSQLExpression class];
}

- (BOOL)isValidQualifierType:(NSString *)typeName model:(EOModel *)model
{
    return ![typeName isEqualToString:@"Data"];
}

- (void)assertConnectionDictionaryIsValid
{
    NSException *exception = nil;
    EOAdaptorContext *adaptorContext;
    EOAdaptorChannel *adaptorChannel;

    if ([self hasOpenChannels])
        return;

    adaptorContext = [self createAdaptorContext];
    adaptorChannel = [adaptorContext createAdaptorChannel];
    NS_DURING
        [adaptorChannel openChannel];
    NS_HANDLER
        exception = localException;
    NS_ENDHANDLER

    [adaptorChannel closeChannel];

    if (exception)
        [exception raise];
}

- (NSArray *)describeDatabaseNamesWithConnectionDictionary:(NSDictionary *)connectionDict
{
    return [MySQLChannel describeDatabaseNamesWithConnectionDictionary:connectionDict];
}

@end
