/*
 * This file is subject to the MiscKit license, see http://www.misckit.org/
 * and search around for details. The bottom line is: you may do what you please
 * with this code, but you're on your own if you run into trouble...
 * Copyright (C) 1999,2000,2001 dmorin,anjo@mac.com
 * Portions of this code derived from the FlatFileAdaptor (C) Apple Computer
 */

#import "common.h"
#import "MySQLChannel.h"
#import "MySQLContext.h"
#import "MySQLAdaptor.h"
#import "MySQLColumn.h"
#import "MySQLSQLExpression.h"
#import "MySQLAdaptor+ExternalTypeMapping.h"
#import <errmsg.h>

@implementation MySQLChannel

- (void)selectAttributes:(NSArray *)attributes fetchSpecification:(EOFetchSpecification *)fetchSpec lock:(BOOL)yn entity:(EOEntity *)entity
{
    NSString *selectString;
    MySQLSQLExpression *sqlExpression;

    //ak: this is commented because it is too verbose
    //if (debugEnabled ) NSLog (@" *** [%@ selectAttributes: %@ %@]\n", self, [attributes description], [fetchSpec description]);

    if (_delegateRespondsTo.shouldSelectAttributes) {
        if (![_delegate adaptorChannel:self shouldSelectAttributes:attributes fetchSpecification:fetchSpec lock:yn entity:entity]) 	{
            return;
        }
    }

    selectString = [entity externalQuery];

    if (!selectString || [fetchSpec qualifier]) {
        //ak: this is commented because it is too verbose
        //if (debugEnabled) NSLog (@" *** [%@ selectAttributes: %@]\n", self, [fetchSpec qualifier]);
        sqlExpression = (MySQLSQLExpression *)[[[_context adaptor] expressionClass] selectStatementForAttributes:attributes lock:yn fetchSpecification:fetchSpec entity:entity];
        
    } else {
        //ak: this is commented because it is too verbose
        // if (debugEnabled) NSLog (@" *** [%@ selectAttributes: %@]\n", self, selectString);
        sqlExpression = (MySQLSQLExpression *)[[[_context adaptor] expressionClass] expressionForString:selectString];
    }

    // changed fetch limit handling dec 7th...
    if( [fetchSpec fetchLimit] && ![fetchSpec promptsAfterFetchLimit])
        [self setFetchLimit:[fetchSpec fetchLimit]];
    else
        [self setFetchLimit:0];

    //ak: this is commented because it is too verbose
    // if (debugEnabled) NSLog (@" *** [%@ selectAttributes: %@]\n", self, [sqlExpression description]);

    [self evaluateExpression:sqlExpression];

    [self setAttributesToFetch:attributes];

    if (_delegateRespondsTo.didSelectAttributes)
        [_delegate adaptorChannel:self didSelectAttributes:attributes fetchSpecification:fetchSpec lock:yn entity:entity];

    return;
}

- (void)_endFetch
{
    if (_flags.isFetchInProgress)
      {
        _flags.isFetchInProgress = NO;
        mysql_free_result(result);
        result = NULL;
      }
    [self setAttributesToFetch:nil];
}

- initWithMySQLContext:(MySQLContext *)context
{
    NSDictionary *connectionDictionary;

    [super initWithAdaptorContext:context];
    mysql = NULL;
    connectionDictionary = [[context adaptor] connectionDictionary];
    useMySQLStoreResult = [[connectionDictionary objectForKey:@"useMySQLStoreResult"] isEqual:@"YES"];
    debugEnabled = [[NSUserDefaults standardUserDefaults] boolForKey:@"EODebuggingEnabled"];
    return self;
}

- (void)dealloc
{

    [super dealloc];
}

- (void)openChannel
{
    NSDictionary *connectionDict = [[_context adaptor] connectionDictionary];
    NSString *user = [connectionDict objectForKey:@"userName"];
    NSString *password = [connectionDict objectForKey:@"password"];
    NSString *hostName = [connectionDict objectForKey:@"hostName"];
    NSString *port = [connectionDict objectForKey:@"port"];
    NSString *dbName = [connectionDict objectForKey:@"databaseName"];
    mysql = mysql_init(NULL);
    if ( mysql && _flags.isOpen == NO )
      {
        if ( !mysql_real_connect(mysql, [hostName cString], [user cString], [password cString], NULL, [port intValue], NULL, 0) )
            [NSException raise:@"MySQLAdaptorExeption" format:@"Open failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];

        if ( mysql_select_db(mysql, [dbName cString]) )
            [NSException raise:@"MySQLAdaptorExeption" format:@"Select DB failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];
        _flags.isOpen = YES;
      }
    else
        [NSException raise:@"MySQLAdaptorExeption" format:@"Init failed."];
}

- (void)closeChannel
{
    if ([self isOpen]) {
        if ([self isFetchInProgress])
            [self cancelFetch];
        mysql_close(mysql);
        _flags.isOpen = NO;
    }
}

- (void)cancelFetch
{
    if( _flags.isFetchInProgress )
      {
        if (debugEnabled) NSLog (@" !!! %@ fetch canceled\n", self);
        [self _endFetch];
      }
}

- (BOOL)isOpen
{
    return _flags.isOpen;
}

- (BOOL)isFetchInProgress
{
    return _flags.isFetchInProgress;
}


- (void)evaluateExpression:(EOSQLExpression *)expression_
{
    NSStringEncoding encoding = [[_context adaptor] databaseEncoding];
    NSData *data;
    NSString *sqlString;
    MySQLSQLExpression *expression = (MySQLSQLExpression *)expression_;
    
    if (_flags.isFetchInProgress) {
        [NSException raise:NSInternalInconsistencyException format:@"%@ -- %@ 0x%x: illegal attempt to evaluateExpression while a fetch is in progress", NSStringFromSelector(_cmd), NSStringFromClass([self class]), self];
    }

    if (_delegateRespondsTo.shouldEvaluateExpression) {
        if (![_delegate adaptorChannel:self shouldEvaluateExpression:expression]) return;
    }


    sqlString = [expression statement];
    if([expression_ isKindOfClass:[MySQLSQLExpression class]] && [self fetchLimit] && [sqlString hasPrefix:@"SELECT"])
    {
        // ak: Simple fix for Fetch limit NSLog(sqlString);
        // This will append "LIMIT <fetchLimit>" if the fetchSpec doesn't intend to get the results after the limit anyway
        sqlString = [sqlString stringByAppendingFormat:@" LIMIT %d",  [self fetchLimit]];
    }
    data = [sqlString dataUsingEncoding:encoding];
     
    // If there are characters in the string that can't be converted,
    // then the data will be nil, call report error and return FAIL.
    if (!data) {
        [NSException raise:NSInvalidArgumentException format:@"%@ -- %@ 0x%x: attempt to evaluate SQL statement that contains characters that can not be converted to target database encoding (%d): %@ ", NSStringFromSelector(_cmd), NSStringFromClass([self class]), self, encoding, sqlString];
    }

    if (debugEnabled) NSLog (@"[%@ - evaluateExpression:] %@\n", self, sqlString);

    if ( mysql_real_query(mysql,(char *)[data bytes], [data length]) != 0)
    {
        int err = mysql_errno(mysql);

        //ak: this is for the rare cases when the server disconnects us because we took too long between two queries
        if(( err == CR_SERVER_GONE_ERROR || err == CR_SERVER_LOST ))
        {
            if (debugEnabled) NSLog(@"Mysql server is gone, trying to reconnect.");

            if([self isFetchInProgress])
                [self cancelFetch];
            if([self isOpen])
                [self closeChannel];

            [self openChannel];
            return [self evaluateExpression:expression];
        }
        [NSException raise:@"MySQLAdaptorExeption" format:@"Execute Query [query] failed with error no: %d\n%s", err, mysql_error(mysql)];
    }

    if ( useMySQLStoreResult )
        result = mysql_store_result(mysql);
    else
        result = mysql_use_result(mysql);

    if(mysql_field_count(mysql) == 0)
        _rowsProcessedCount = (unsigned long)mysql_affected_rows(mysql);

    if ( !result && mysql_errno(mysql) )
        [NSException raise:@"MySQLAdaptorExeption" format:@"Execute Query [store] failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];
    //ak: this is commented because it is too verbose
    //if (debugEnabled) NSLog(@"*** %@ evaluateExpression: affected rows:%d, result:0x%x, num_rows:%d", self, (unsigned long)_rowsProcessedCount, result, result?(unsigned long)mysql_num_rows(result):0,sqlString);

    if ( result )
        _flags.isFetchInProgress = YES;

    if (_delegateRespondsTo.didEvaluateExpression)
        [_delegate adaptorChannel:self didEvaluateExpression:expression];

    return;
}

- (NSMutableDictionary *)fetchRowWithZone:(NSZone *)zone
{
    MYSQL_ROW sqlRow;
    NSMutableDictionary *row = nil;
    if (![self isFetchInProgress])
        return nil; // Should be an error ?

    // Check if it's the first row, and in this case
    // verify that the attribute set is here
    if(!_attributes)
      {
        [self setAttributesToFetch:[self describeResults]];
    }

    if (_delegateRespondsTo.willFetchRow) {
        [_delegate adaptorChannelWillFetchRow:self];
    }
    sqlRow = mysql_fetch_row (result);
    if( !sqlRow )
      {
        if ( mysql_errno(mysql) )
            [NSException raise:@"MySQLAdaptorExeption" format:@"Fetch Row failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];
        
        [self _endFetch];
        if (_delegateRespondsTo.didFinishFetching) {
            [_delegate adaptorChannelDidFinishFetching:self];
        }
        return nil;
    }

    {
        int i, columnCount;
        id valueBuffer[100];
        id *values, *sp, *ep;
        unsigned long *lengths; //[100];

        lengths = mysql_fetch_lengths(result);
        
        // Create array for values if 100 is too small
        columnCount = [_attributes count];//[_selectedColumns count];
        values = (columnCount > 100) ? (id *)NSZoneMalloc(NULL, columnCount * sizeof(id)) : valueBuffer;

        // Fetch values for row
        for (i = 0; i < columnCount; i++)
          {
            if ( !(sqlRow[i]) )
                values[i] = [EONull null];
            else
              {
                values[i] = [[_selectedColumns objectAtIndex:i] newValueForBytes:sqlRow[i] length:lengths[i]];
              }
               // values[i] = [[_attributes objectAtIndex:i] newValueForBytes:sqlRow[i] length:lengths[i] encoding:[[_context adaptor] databaseEncoding]];
                
          }
        // construct a dictionary with the values
        row = [self dictionaryWithObjects:values forAttributes:_attributes zone:zone];
        // release all of the objects
        for(sp = values, ep = sp + columnCount; sp < ep; sp++)
            [*sp release];
        // relase values if we allocated a value buffer
        if (values != valueBuffer) NSZoneFree(NULL, values);

    }
    if(row && _delegateRespondsTo.didFetchRow) {
        [_delegate adaptorChannel:self didFetchRow:row];
    }

    return row;
}

- (void)insertRow:(NSDictionary *)row forEntity:(EOEntity *)entity
{
    [self evaluateExpression:[[[_context adaptor] expressionClass] insertStatementForRow:row entity:entity]];
}

- (unsigned)updateValues:(NSDictionary *)row inRowsDescribedByQualifier:(EOQualifier *)qualifier entity:(EOEntity *)entity
{
    [self evaluateExpression:[[[_context adaptor] expressionClass] updateStatementForRow:row qualifier:qualifier entity:entity]];
    return _rowsProcessedCount == 0? 1:_rowsProcessedCount;
}

- (unsigned)deleteRowsDescribedByQualifier:(EOQualifier *)qualifier entity:(EOEntity *)entity
{
    [self evaluateExpression:[[[_context adaptor] expressionClass] deleteStatementWithQualifier:qualifier entity:entity]];
    return _rowsProcessedCount;
}


- (NSArray *)describeResults
{
    NSMutableArray *attributes  = [NSMutableArray array];
//    unsigned int numberOfFields;
//    int column;
    MYSQL_FIELD *field;

    // Get number of columns in result set
//    numberOfFields = mysql_num_fields (result);
    mysql_field_seek(result, 0);
    while ( field = mysql_fetch_field(result) )
      {
        NSString *type;
        NSString* name = [NSString stringWithCString:field->name];
        EOAttribute *attribute = [[[EOAttribute alloc] init] autorelease];
        [attribute setName:name];
        [attribute setColumnName:name];
        [attributes addObject:attribute];
        type = [[[_context adaptor] class] externalTypeForMySQLType:field->type];
        [attribute setExternalType:type];
        [attribute setValueClassName:[[[[_context adaptor] class] externalToInternalTypeMap] objectForKey:type]];
      }
    return attributes;
}

- (void)setAttributesToFetch:(NSArray *)attributes
{
    int i, count = 0;

    [_attributes autorelease];
    _attributes = [attributes retain];
 
    count = [_attributes count];
    if(_selectedColumns) {
        [_selectedColumns removeAllObjects];
    } else if(count) {
        _selectedColumns = [[NSMutableArray alloc] initWithCapacity:count];
    }

    // Create MySQLColumns
    for (i = 0; i < count; i++) {
        EOAttribute *attribute;
        MySQLColumn *column;

        // Create column for selected attribute
        attribute = [_attributes objectAtIndex:i];
        column = [MySQLColumn columnForAttribute:attribute];
        [_selectedColumns addObject: column];
   }
}

- (NSArray *)attributesToFetch
{
    return _attributes;
}

- (NSArray *)_entityGroupForEntity:(EOEntity *)entity
{
    NSMutableArray *resultArray = [NSMutableArray array];
    NSMutableArray *tableNames = [NSMutableArray array];
    NSArray *subEntities = [entity subEntities];

    if(![entity isAbstractEntity] && [entity externalName]) {
        [resultArray addObject:entity];
        [tableNames addObject:[entity externalName]];
    }
    if(subEntities)
      {
        unsigned i,c;

        for(i = 0, c = [subEntities count]; i < c; i++)
          {
            EOEntity *sub = [subEntities objectAtIndex:i];
            NSArray *merge = [self _entityGroupForEntity:sub];
            unsigned j,k;

            for(j = 0, k = [merge count]; j < k; j++)
              {
                unsigned index;
                EOEntity *ent = [merge objectAtIndex:j];

                index = [tableNames indexOfObject:[ent externalName]];
                if(index == NSNotFound)
                  {
                    [resultArray addObject:ent];
                    [tableNames addObject:[ent externalName]];
                  }
              }
          }
      }
    return resultArray;
}

- (NSDictionary *)primaryKeyForNewRowWithEntity:(EOEntity *)entity
{
    NSArray *primaryKeyAttributes = [entity primaryKeyAttributes];
    EOAttribute *primAttribute;
    Class expressionClass;
    EOSQLExpression *getRowExpr, *lockExpr, *unlockExpr;
    EOSQLExpression *updateRowExpr;
    NSDictionary *row = nil;
    NSNumber *returnedValue, *newValue = nil;
    BOOL needToCreateTable = NO;

    if([primaryKeyAttributes count] != 1) {
        // We support only simple primary keys, data keys are handled by superclass
        return [super primaryKeyForNewRowWithEntity:entity];
    }

    primAttribute = [primaryKeyAttributes objectAtIndex:0];

    if([primAttribute adaptorValueType] != EOAdaptorNumberType) {
        // We support only number keys, so call the superclass
         return [super primaryKeyForNewRowWithEntity:entity];
   }

    expressionClass = [[[self adaptorContext] adaptor] expressionClass];
    //ak: lock the involved table and the EO_PK_TABLE
    // This will of course fail if the PK_TABLE doesn't exist
    lockExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"LOCK TABLE EO_PK_TABLE WRITE, %@ READ", [entity externalName]]];

    //ak: Create an unlock statement for later use
    unlockExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"UNLOCK TABLES"]];

    NS_DURING
        // Lock PK table and target table if it exists
        [self evaluateExpression:lockExpr];
    NS_HANDLER
        needToCreateTable = YES;
    NS_ENDHANDLER

    if(needToCreateTable)
    {
        EOSQLExpression *createTableExpr;

        createTableExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"CREATE TABLE EO_PK_TABLE ( NAME CHAR(40), PK INT )"]];

        // Create the table
        NS_DURING
            [self evaluateExpression:createTableExpr];
        NS_HANDLER
            [localException raise];
        NS_ENDHANDLER

        // Lock PK table and target table if it exists (second try)
        NS_DURING
            [self evaluateExpression:lockExpr];
        NS_HANDLER
            // something is really fishy here, bail out
            [localException raise];
        NS_ENDHANDLER
    }

    getRowExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"SELECT PK FROM EO_PK_TABLE WHERE NAME = '%@'", [entity externalName]]];

    // Fetch the row
    NS_DURING
        [self evaluateExpression:getRowExpr];
        row = [self fetchRowWithZone:NULL]; // fetch one row
        [self cancelFetch];
    NS_HANDLER
        [self cancelFetch];
    NS_ENDHANDLER

    if(row)
    {
        if(debugEnabled) NSLog(@"primaryKeyForNewRowWithEntity: Got pk: %@", row);
        returnedValue = [row objectForKey:@"PK"];
    }
    else
    {
        EOSQLExpression *insertRowExpr;

        // Insert a row with 0
        insertRowExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"INSERT INTO EO_PK_TABLE(NAME, PK) VALUES ( '%@', 0 )", [entity externalName]]];

        NS_DURING
            [self evaluateExpression:insertRowExpr];
        NS_HANDLER
            //[[self adaptorContext] rollbackTransaction];
            [self evaluateExpression:unlockExpr];
            [localException raise];
        NS_ENDHANDLER

        returnedValue = [NSNumber numberWithInt:0];
    }

    if((id)returnedValue == [EONull null] || ![returnedValue intValue])
    {
        // Find the current maximum, and set it as the newValue...
        NSArray *entityGroup = [self _entityGroupForEntity:entity];
        unsigned i,c;

        // NSLog(@"primaryKeyForNewRowWithEntity: Setting MAX in EO_PK_TABLE");

        newValue = [NSNumber numberWithInt:0];

        for(i = 0, c = [entityGroup count]; i < c; i++) {
            EOEntity *currentEntity = [entityGroup objectAtIndex:i];
            EOAttribute *att = [currentEntity attributeNamed:[primAttribute name]];
            NSDictionary *r;
            NSNumber *tempValue;
            EOSQLExpression *searchExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"SELECT MAX(%@) FROM %@", [att columnName], [currentEntity externalName]]];

            // Fetch the row
            NS_DURING
                [self evaluateExpression:searchExpr];
                [self setAttributesToFetch:[NSArray arrayWithObject:att]];
                r = [self fetchRowWithZone:NULL]; // fetch one row
                [self cancelFetch];
            NS_HANDLER
                [self cancelFetch];
                [self evaluateExpression:unlockExpr];
                [localException raise];
            NS_ENDHANDLER

            tempValue = [r objectForKey:[att name]];
            if(tempValue && ((id)tempValue != [EONull null]) && ([tempValue compare:newValue] == NSOrderedDescending))
                newValue = tempValue;
        }
    }
    else
    {
        newValue = returnedValue;
    }
    newValue = [NSNumber numberWithInt:[newValue intValue] + 1];

    // Update the table
    updateRowExpr = [expressionClass expressionForString:[NSString stringWithFormat:@"UPDATE EO_PK_TABLE SET PK = %d WHERE NAME = '%@' AND PK = %d", [newValue intValue], [entity externalName], [returnedValue intValue]]];
   
    NS_DURING
        [self evaluateExpression:updateRowExpr];
    NS_HANDLER
        [self evaluateExpression:unlockExpr];
        [localException raise];
    NS_ENDHANDLER

    if(_rowsProcessedCount != 1L) {                                                                                                                                                                               	// Somebody did ask for a key at the same time !
        // Redo it ?
        // ak: this should not happen as we lock the tables...
        // however, this whole scheme fails if the data is manipulated outside of EOF and the PK table isn't updated.
        [self evaluateExpression:unlockExpr];
        return nil;
    }
    [self evaluateExpression:unlockExpr];
    return [NSDictionary dictionaryWithObjectsAndKeys:newValue, [primAttribute name], nil];
}
@end

@implementation MySQLChannel(Private)

- (unsigned int)fetchLimit
{
    return _fetchLimit;
}

- (void)setFetchLimit:(unsigned int)newFetchLimit
{
    _fetchLimit=newFetchLimit;

    return;
}

@end

