/*
 * This file is subject to the MiscKit license, see http://www.misckit.org/
 * and search around for details. The bottom line is: you may do what you please
 * with this code, but you're on your own if you run into trouble...
 * Copyright (C) 1999,2000,2001 dmorin,anjo@mac.com
 * Portions of this code derived from the FlatFileAdaptor (C) Apple Computer
 */

#import "common.h"
#import "MySQLColumn.h"
#import <EOAccess/EOAccess.h>

@implementation MySQLColumn

+ (Class)columnClassForAttribute:(EOAttribute *)attr
{
    switch ([attr adaptorValueType]) {
        case EOAdaptorNumberType : return [MySQLNumberColumn class];
        case EOAdaptorCharactersType : return [MySQLStringColumn class];
        case EOAdaptorBytesType : return [MySQLDataColumn class];
        case EOAdaptorDateType : return [MySQLDateColumn class];
    }

    return nil;
}

+ columnForAttribute:(EOAttribute *)attr
{
    return [[[[self columnClassForAttribute:attr] alloc] initWithAttribute:attr type:0] autorelease];
}

- initWithAttribute:(EOAttribute *)attr type:(int)type
{
    [super init];
    attribute = [attr retain];
    fieldType = type;
    
    return self;
}

- (EOAttribute *)attribute
{
    return attribute;
}

- (void)dealloc
{
    [attribute release];
    [super dealloc];
}

- (id)newValueForBytes:(const void *)bytes length:(int)length
{
    return nil;
}

@end

@implementation MySQLNumberColumn

- initWithAttribute:(EOAttribute *)attr type:(int)type
{
    [super initWithAttribute:attr type:type];

    if ([[attribute valueClassName] isEqualToString:@"NSDecimalNumber"])
        valueType = EODecimalNumberType;
    else if ([[attribute valueType] isEqualToString:@"i"])
        valueType = EOIntegerType;
    else
        valueType = EODoubleType;

    return self;
}

- (id)newValueForBytes:(const void *)bytes length:(int)length
{
    NSString *helper;
    id value;

    if ( !bytes ) return [EONull null];
    helper = [[NSString alloc] initWithCString:(char *)bytes length:length];

    switch (valueType) {
        case EOIntegerType :
            value = [[NSNumber alloc] initWithInt:[helper intValue]];
            break;
        case EODoubleType :
            value = [[NSNumber alloc] initWithDouble:[helper doubleValue]];
            break;
        default :
            value = [[NSDecimalNumber decimalNumberWithString:helper] retain];
    }

    [helper release];
    return value;
}

@end

@implementation MySQLStringColumn

- initWithAttribute:(EOAttribute *)attr type:(int)type
{
    [super initWithAttribute:attr type:type];
    encoding = NSISOLatin1StringEncoding;

    return self;
}

- (id)newValueForBytes:(const void *)bytes length:(int)length
{
    if ( !bytes ) return [EONull null];
    return [attribute newValueForBytes:bytes length:length encoding:encoding];
}

@end

@implementation MySQLDataColumn

- (id)newValueForBytes:(const void *)bytes length:(int)length
{
    if ( !bytes ) return [EONull null];
    return [attribute newValueForBytes:bytes length:length];
}

@end

@implementation MySQLDateColumn

- (id)newValueForBytes:(const void *)bytes length:(int)length
{
    NSScanner *scanner;
    NSString *stringToScan;
    int year, month, day, hour, minute, second;
//    NSString *dateFormat = @"%Y-%m-%d %H:%M:%S";

    if ( !bytes ) return [EONull null];
    stringToScan = [NSString stringWithCString:bytes length:length];
    scanner = [NSScanner scannerWithString:stringToScan];

    if(length == 14)
     {
         NSCalendarDate *date = [[NSCalendarDate dateWithString:stringToScan calendarFormat:@"%Y%m%d%H%M%S"] retain];
         return date;
     }
    // The is a date to scan
    if ( length >= 10 )
      {
        [scanner scanInt:&year];
        [scanner scanString:@"-" intoString:NULL];
        [scanner scanInt:&month];
        [scanner scanString:@"-" intoString:NULL];
        [scanner scanInt:&day];
      }
    else
      {
        year = 0; month = 0; day = 0;
      }
    // if complete date, skip space
    if ( length > 10 )
        [scanner scanString:@" " intoString:NULL];
    // if time is in there
    if ( length != 10 )
      {
        [scanner scanInt:&hour];
        [scanner scanString:@":" intoString:NULL];
        [scanner scanInt:&minute];
        [scanner scanString:@":" intoString:NULL];
        [scanner scanInt:&second];
      }
    else
    {
        hour = 0; minute = 0; second = 0;
    }
     return [attribute newDateForYear:year
                               month:month
                                 day:day
                                hour:hour
                              minute:minute
                              second:second
                         millisecond:0
                            timezone:nil
                                zone:0];

    /*
    switch ( fieldType )
      {
        case FIELD_TYPE_TIMESTAMP:
            dateFormat = @"%Y-%m-%d %H:%M:%S";
            break;
        case FIELD_TYPE_DATE:
            dateFormat = @"%Y-%m-%d";
            break;
        case FIELD_TYPE_TIME:
            dateFormat = @"%H:%M:%S";
            break;
        case FIELD_TYPE_DATETIME:
            dateFormat = @"%Y-%m-%d %H:%M:%S";
            break;
        case FIELD_TYPE_NEWDATE:
            dateFormat = @"%Y-%m-%d %H:%M:%S";
            break;
      }
   */
}

@end
