/*
 * This file is subject to the MiscKit license, see http://www.misckit.org/
 * and search around for details. The bottom line is: you may do what you please
 * with this code, but you're on your own if you run into trouble...
 * Copyright (C) 1999,2000,2001 dmorin,anjo@mac.com
 * Portions of this code derived from the FlatFileAdaptor (C) Apple Computer
 */

#import "MySQLContext.h"
#import "MySQLAdaptor.h"
#import "MySQLChannel.h"

@implementation MySQLContext

- initWithMySQLAdaptor:(MySQLAdaptor *)adaptor
{
    [super initWithAdaptor:adaptor];
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)beginTransaction
{
 //   if ([self transactionNestingLevel]) {
  //      [NSException raise:NSInternalInconsistencyException format:@"%@ -- %@ 0x%x: attempted to begin a transaction within a transaction", NSStringFromSelector(_cmd), NSStringFromClass([self class]), self];
   // }

    if (_delegateRespondsTo.shouldBegin) {
        if (![_delegate adaptorContextShouldBegin:self]) return;
    }

    // a normal adaptor would actually do something to begin the transaction here

    [self transactionDidBegin];

    if (_delegateRespondsTo.didBegin)
        [_delegate adaptorContextDidBegin:self];
}

- (void)commitTransaction
{
    if (![self transactionNestingLevel])
        [NSException raise:NSInternalInconsistencyException format:@"%@ -- %@ 0x%x:illegal attempt to commit a transaction when there are none in progress", NSStringFromSelector(_cmd), NSStringFromClass([self class]), self];

    if (_delegateRespondsTo.shouldCommit) {
        if (![_delegate adaptorContextShouldCommit:self]) return;
    }

// Commiting
//    [self saveDirtyTables];
    [self transactionDidCommit];

    if (_delegateRespondsTo.didCommit)
        [_delegate adaptorContextDidCommit:self];
}

- (void)rollbackTransaction
{
    if (![self transactionNestingLevel])
        [NSException raise:NSInternalInconsistencyException format:@"%@ -- %@ 0x%x: attempted to commit a transaction when there are none in progress", NSStringFromSelector(_cmd), NSStringFromClass([self class]), self];

    if (_delegateRespondsTo.shouldRollback) {
        if (![_delegate adaptorContextShouldRollback:self]) return;
    }

    // Roll Back
//    [self dropDirtyTables];
    [self transactionDidRollback];

    if (_delegateRespondsTo.didRollback)
        [_delegate adaptorContextDidRollback:self];
}

- (BOOL)canNestTransactions
{
    return NO;
}

- (EOAdaptorChannel *)createAdaptorChannel
{
    return [[[MySQLChannel alloc] initWithMySQLContext:self] autorelease];
}

/*
- (BOOL)autoBeginTransaction {
    if (![self transactionNestingLevel]) {
        [self beginTransaction];
        didAutoBegin = YES;
        return YES;
    }
    return NO;
}

- (BOOL)autoCommitTransaction {
    if (didAutoBegin) {
        [self commitTransaction];
        didAutoBegin = NO;
        return YES;
    }
    return NO;
}
*/

@end
