/*
 * This file is subject to the MiscKit license, see http://www.misckit.org/
 * and search around for details. The bottom line is: you may do what you please
 * with this code, but you're on your own if you run into trouble...
 * Copyright (C) 1999,2000,2001 dmorin,anjo@mac.com
 * Portions of this code derived from the FlatFileAdaptor (C) Apple Computer
 */

#import "common.h"
#import "MySQLDescription.h"
#import "MySQLAdaptor+ExternalTypeMapping.h"

@implementation MySQLChannel (ModelDescription)

- (NSArray *)describeTableNames
{
    MYSQL_RES *localResult;
    MYSQL_ROW	row ;
    NSMutableArray *tablenames;

    if (debugEnabled) NSLog (@" *** [%@ describeTableNames]\n", self);
    
    localResult = mysql_list_tables(mysql, NULL);
    if (!localResult)
        [NSException raise:@"MySQLAdaptorExeption" format:@"Describe Table Names failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];

    tablenames = [NSMutableArray arrayWithCapacity:(unsigned long)mysql_num_rows( localResult )];

    while ( row = mysql_fetch_row( localResult ) )
      {
        if([[NSString stringWithCString:row[0]] hasPrefix:@"EO_"])
            continue;
        [tablenames addObject:[NSString stringWithCString:row[0]]];
      }

    mysql_free_result(localResult);
    return tablenames;
}


- (EOModel *)describeModelWithTableNames:(NSArray *)tableNames
{
    EOAdaptor *adaptor;
    EOModel *model = [[EOModel new] autorelease];
    EOEntity *entity;
    NSString *name;
    int i, tableCount = [tableNames count];

    adaptor = [[self adaptorContext] adaptor];
    [model setAdaptorName:[adaptor name]];
    [model setConnectionDictionary:[adaptor connectionDictionary]];

    for (i = 0; i < tableCount; i++) {
        name = [tableNames objectAtIndex:i];
        entity = [EOEntity new];
        [entity setName:name];
        [entity setExternalName:name];
        [model addEntity:entity];
        [self describeEntity:entity];
    }
    return model;
}

- (void)describeStoredProceduresForModel:(EOModel *)model
{
    return;
}

- (void)describeEntity:(EOEntity *)entity
{
    MYSQL_RES *localResult;
    MYSQL_ROW	row ;
    EOAdaptor *adaptor = [[self adaptorContext] adaptor];

    if (debugEnabled) NSLog (@" *** [%@ describeEntity:%@]\n", self, [entity externalName]);

    //result = mysql_list_fields(mysql, [tableName cString], NULL);
    if ( mysql_query(mysql, [[@"show columns from " stringByAppendingString:[entity externalName]] cString]) )
        [NSException raise:@"MySQLAdaptorExeption" format:@"Describe list_fields failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];

    localResult = mysql_store_result(mysql);
    if (!localResult)
        [NSException raise:@"MySQLAdaptorExeption" format:@"Describe list_fields failed with error no: %d\n%s", mysql_errno(mysql), mysql_error(mysql)];
    
    while ( row = mysql_fetch_row( localResult ) )
      {        
        NSString *typeCol, *type, *name, *allowNull, *index;
        EOAttribute *attribute = [[EOAttribute new] autorelease];

        name = [NSString stringWithCString:row[0]];
        typeCol = [NSString stringWithCString:row[1]];
        allowNull = [NSString stringWithCString:row[2]];
        index = [NSString stringWithCString:row[3]];
        
        [attribute setName:name];
        [attribute beautifyName];
        [attribute setColumnName:name];

        type = [[[typeCol componentsSeparatedByString:@"("] objectAtIndex:0] uppercaseString];
        [attribute setExternalType:type];
        [attribute setValueClassName:[[[adaptor class] externalToInternalTypeMap] objectForKey:type]];

        if ( [type isLike:@"*INT*"] || [type isLike:@"*TIMESTAMP*"])
            [attribute setValueType:@"i"];

        if ( [type isLike:@"*DOUBLE*"] || [type isLike:@"*FLOAT*"] || [type isLike:@"*DECIMAL*"])
        {
            int length;
            NSScanner *scanner = [NSScanner scannerWithString:typeCol];
            [attribute setValueType:@"f"];
            [scanner setScanLocation:[type length]+1];
            if ( [scanner scanInt:&length] )
                [attribute setScale:length];
            [scanner scanString:@"," intoString:NULL];
            if ( [scanner scanInt:&length] )
                [attribute setPrecision:length];
        }
        [attribute setAllowsNull:[allowNull isEqual:@"YES"]];

        if ( [type isLike:@"*CHAR"] )
          {
            int length;
            NSScanner *scanner = [NSScanner scannerWithString:typeCol];
            [scanner setScanLocation:[type length]+1]; 
            if ( [scanner scanInt:&length] )
                [attribute setWidth:length];
          }

        [entity addAttribute:attribute];
        
        if ( [index isLike:@"PRI"] )
          {
            NSMutableArray *primaryKeys;

            primaryKeys = [[entity primaryKeyAttributes] mutableCopy];
            if ( !primaryKeys )
                primaryKeys = [NSMutableArray array];
            [primaryKeys addObject:attribute];
            [entity setPrimaryKeyAttributes:primaryKeys];
          }
      }

    mysql_free_result(localResult);
}

+ (NSArray *)describeDatabaseNamesWithConnectionDictionary:(NSDictionary *)connectionDict
{
    MYSQL *localMysql;
    MYSQL_RES *localResult;
    MYSQL_ROW	row ;
    NSMutableArray *databaseNames;

    NSString *user = [connectionDict objectForKey:@"userName"];
    NSString *password = [connectionDict objectForKey:@"password"];
    NSString *hostName = [connectionDict objectForKey:@"hostName"];
    NSString *port = [connectionDict objectForKey:@"port"];

    localMysql = mysql_init(NULL);
    if ( !localMysql )
        [NSException raise:@"MySQLAdaptorExeption" format:@"Init failed."];

    if ( !mysql_real_connect(localMysql, [hostName cString], [user cString], [password cString], NULL, [port intValue], NULL, 0) )
        [NSException raise:@"MySQLAdaptorExeption" format:@"Open failed with error no: %d\n%s", mysql_errno(localMysql), mysql_error(localMysql)];

    localResult = mysql_list_dbs(localMysql, NULL);
    if (!localResult)
        [NSException raise:@"MySQLAdaptorExeption" format:@"Describe Databases Names failed with error no: %d\n%s", mysql_errno(localMysql), mysql_error(localMysql)];

    databaseNames = [NSMutableArray arrayWithCapacity:(unsigned long)mysql_num_rows( localResult )];

    while ( row = mysql_fetch_row( localResult ) )
      {
        [databaseNames addObject:[NSString stringWithCString:row[0]]];
      }
    mysql_free_result(localResult);
    mysql_close(localMysql);
    return databaseNames;
}

@end
